import AppKit

// Dummy menu for NSStatusItem which allows to asynchronously
// execute callback when it's being opened. This gives us an
// possibilty to load other menu in a non-blocking manner.
// See Maccy.withFocus() for more details about why this is needed.
class MenuLoader: NSMenu, NSMenuDelegate {
  typealias LoaderCallback = () -> Void
  private var loader: LoaderCallback!

  required init(coder decoder: NSCoder) {
    super.init(coder: decoder)
  }

  init(_ loader: @escaping LoaderCallback) {
    super.init(title: "Loader")
    addItem(withTitle: "Loading...", action: nil, keyEquivalent: "")
    self.delegate = self
    self.loader = loader
  }

  func menuWillOpen(_ menu: NSMenu) {
    menu.cancelTracking()
    // Just calling loader() doesn't work when avoidTakingFocus is true.
    Timer.scheduledTimer(withTimeInterval: 0.01, repeats: false) { _ in
      self.loader()
    }
  }
}
